ENV['RACK_ENV'] = 'test'
require 'helper'
require 'rack/test'
require 'sinatra/base'

class TestApp < Sinatra::Base
  use Seojs::Middleware
  
  get '/hi' do
    "Hello!"
  end
end

class TestMiddleware < Test::Unit::TestCase
  include Rack::Test::Methods

  def app
    TestApp.new
  end
  
  def setup
    Seojs.token = nil
  end
  
  def test_hi
    get '/hi'
    assert last_response.ok?
    assert_equal "Hello!", last_response.body
  end
  
  def test_its_a_trap
    get '/trap'
    assert !last_response.ok?
  end
  
  def test_forward_without_token
    Seojs.token = nil
    Net::HTTP::Get.expects(:new).never
    get '/hi', :_escaped_fragment_ => ''
    assert last_response.ok?
  end
  
  def build_response(body, code, headers)
     r = Net::HTTPResponse.new(1.0, code, "body")
     headers.each_pair do |k,v|
       r.add_field(k,v)
     end
     r.stubs(:body => body)
     r
  end
  
  def test_forward_with_token
    Seojs.token = 'token'
    Net::HTTP::Get.expects(:new)
                  .with(equals("http://cdn.getseojs.com/snapshots/token/v3/http://example.org:80/hi?_escaped_fragment_="))
    Net::HTTP.expects(:start)
             .returns(build_response('Snapshot!', 200, {}))
    get '/hi', :_escaped_fragment_ => ''
    assert last_response.ok?
    assert_equal 'Snapshot!', last_response.body
  end
  
  def test_forward_with_token_timed_out
    Seojs.token = 'token'
    Timeout.expects(:timeout).raises(Timeout::Error)
    get '/hi', :_escaped_fragment_ => ''
    assert last_response.ok?
    assert_equal 'Hello!', last_response.body
  end
  
  def test_forward_headers_with_token
    Seojs.token = 'token'
    Net::HTTP::Get.expects(:new)
                  .with(equals("http://cdn.getseojs.com/snapshots/token/v3/http://example.org:80/hi?_escaped_fragment_="))
                  .returns({})
    Net::HTTP.expects(:start)
             .returns(build_response('Snapshot!', 200, {'ETAG'=>'abc'}))
    get '/hi', {:_escaped_fragment_ => ''}, {'HTTP_IF_NONE_MATCH' => '123'}
    assert_equal 'abc', last_response.headers['ETAG']
    assert last_response.ok?
    assert_equal 'Snapshot!', last_response.body
  end
  
  def test_forward_user_agent
    Seojs.token = 'token'
    request = {}
    request.expects(:[]=).with('USER-AGENT', 'Googlebot')
    Net::HTTP::Get.expects(:new)
                  .with(equals("http://cdn.getseojs.com/snapshots/token/v3/http://example.org:80/hi?_escaped_fragment_="))
                  .returns(request)
    Net::HTTP.expects(:start)
             .returns(build_response('Snapshot!', 200, {'ETAG'=>'abc'}))
    get '/hi', {:_escaped_fragment_ => ''}, {'HTTP_USER_AGENT' => 'Googlebot'}
    assert_equal 'abc', last_response.headers['ETAG']
    assert last_response.ok?
    assert_equal 'Snapshot!', last_response.body
  end

  def test_fallback_if_not_found
    Seojs.token = 'token'
    Net::HTTP::Get.expects(:new)
                  .with(equals("http://cdn.getseojs.com/snapshots/token/v3/http://example.org:80/hi?_escaped_fragment_="))
    Net::HTTP.expects(:start)
             .returns(build_response('not found...', 404, {}))
    get '/hi', :_escaped_fragment_ => ''
    assert last_response.ok?
    assert_equal 'Hello!', last_response.body
  end
  
  def test_forward_redirect
    Seojs.token = 'token'
    Net::HTTP::Get.expects(:new)
                  .with(equals("http://cdn.getseojs.com/snapshots/token/v3/http://example.org:80/hi?_escaped_fragment_="))
    Net::HTTP.expects(:start)
             .returns(build_response('somewhere else', 302, {'Location' => 'http://google.com/'}))
    get '/hi', :_escaped_fragment_ => ''
    assert last_response.redirect?
    follow_redirect!
    assert_equal last_request.url, 'http://google.com/'
  end
  
  def test_test_url
    get '/seojs/test'
    assert last_response.ok?
    assert_equal 'SEO.js Middleware Found!', last_response.body
  end
  
  def test_set_url
    Seojs.url = "abc"
    assert_equal "abc", Seojs.url
  end

end